package org.menacheri.jetserver.concurrent;

import java.util.concurrent.ArrayBlockingQueue;


public class DefaultEventLoop implements
		EventLoop<ArrayBlockingQueue<Command>, Thread>
{
	private final ArrayBlockingQueue<Command> eventQueue;
	private final Thread thread;

	public DefaultEventLoop(ArrayBlockingQueue<Command> eventQueue,
			Thread thread)
	{
		super();
		this.eventQueue = eventQueue;
		this.thread = thread;
	}

	public void publish(Command command)
	{
		eventQueue.add(command);
	}
	
	@Override
	public synchronized void start()
	{
		if (!thread.isAlive())
		{
			thread.start();
		}
	}
	
	@Override
	public ArrayBlockingQueue<Command> getQueue()
	{
		return eventQueue;
	}

	@Override
	public Thread getConsumer()
	{
		return thread;
	}

	@Override
	public void setQueue(ArrayBlockingQueue<Command> queue)
	{
		throw new UnsupportedOperationException(
				"queue is a final variable and cannot be set");
	}

	@Override
	public void setConsumer(Thread consumer)
	{
		throw new UnsupportedOperationException(
				"thread is a final variable and cannot be set");
	}


}
