package org.menacheri.jetserver.concurrent;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadFactory;


public class EventLoopFactory
{
	public static EventLoop<ArrayBlockingQueue<Command>, Thread> newEventLoop(ThreadFactory threadFactory)
	{
		ArrayBlockingQueue<Command> eventQueue = new ArrayBlockingQueue<Command>(2048);
		Thread thread = threadFactory.newThread(new Consumer(eventQueue));
		return new DefaultEventLoop(eventQueue, thread);
	}
	
	public static class Consumer implements Runnable
	{
		private final ArrayBlockingQueue<Command> queue;
		
		public Consumer(ArrayBlockingQueue<Command> queue)
		{
			this.queue = queue;
		}
		
		@Override
		public void run()
		{
			while(true)
			{
				Command command;
				try
				{
					command = queue.take();
				}
				catch (InterruptedException e)
				{
					throw new RuntimeException(e);
				}
				command.execute();
			}
			
		}
	}
}
